#include "Eigen/Dense"
#include <iostream>
#include <cassert>

template<typename T, int M>
Eigen::Matrix<T,M,1> g(const Eigen::Matrix<T,M,1> &x) { return x; }

template<typename T, int M>
T Givens(Eigen::Matrix<T,M,1> x, Eigen::Matrix<T,M,1> y) {
  auto m=x.size();
  for (auto i=m-2;i>=0;i--) {
    Eigen::Matrix<T,M,M> G=Eigen::Matrix<T,M,M>::Identity(m,m);
    T norm_a_tilde=x.block(i,0,2,1).norm();
    G(i,i)=x(i)/norm_a_tilde; G(i+1,i+1)=G(i,i);
    G(i,i+1)=x(i+1)/norm_a_tilde; G(i+1,i)=-G(i,i+1);
    std::cout << "G_" << i+1 << "=" << std::endl << G << std::endl;
    x=G*x; y=G*y; 
    std::cout << "x^T=(Gx)^T=" << x.transpose() << std::endl;
    std::cout << "y^T=(Gy)^T=" << y.transpose() << std::endl << std::endl;
  }
  return y(0)/x(0);
}

int main(int argc, char* argv[]) {
  assert(argc==2); auto M=std::stoi(argv[1]);
  using T=float; 
  using VT=Eigen::Matrix<T,Eigen::Dynamic,1>;
  VT x=VT::Random(M), y=VT::Random(M); 
  x=x.cwiseProduct(x);
  std::cout << "x^T=" << x.transpose() << std::endl
            << "y^T=" << y.transpose() << std::endl;
  T p=Givens(g(x),y);
  std::cout << "p=" << p << std::endl;
  return 0;
}
